# frozen_string_literal: true

class Hyrax::ValkyrieUpload
  # @param [IO] io
  # @param [String] filename
  # @param [Hyrax::FileSet] file_set
  # @param [Valkyrie::StorageAdapter] storage_adapter
  # @param [RDF::URI] use
  # @param [User] user
  #
  # @see Hyrax::FileMetadata::Use
  # @return [Hyrax::FileMetadata] the metadata representing the uploaded file
  # rubocop:disable Metrics/MethodLength
  # rubocop:disable Metrics/ParameterLists
  def self.file(
    filename:,
    file_set:,
    io:,
    storage_adapter: Hyrax.storage_adapter,
    use: Hyrax::FileMetadata::Use::ORIGINAL_FILE,
    user: nil
  )
    new(storage_adapter: storage_adapter)
      .upload(filename: filename, file_set: file_set, io: io, use: use, user: user)
  end

  ##
  # @!attribute [r] storage_adapter
  #   @return [Valkyrie::StorageAdapter] storage_adapter
  attr_reader :storage_adapter
  ##
  # @param [Valkyrie::StorageAdapter] storage_adapter
  # @param [Class] file_set_file_service implementer of {Hyrax::FileSetFileService}
  def initialize(storage_adapter: Hyrax.storage_adapter, file_set_file_service: Hyrax.config.file_set_file_service)
    @storage_adapter = storage_adapter
    @file_set_file_service = file_set_file_service
  end

  def upload(filename:, file_set:, io:, use: Hyrax::FileMetadata::Use::ORIGINAL_FILE, user: nil, mime_type: nil) # rubocop:disable Metrics/AbcSize
    return build_file_metadata(filename: filename, file_set: file_set, user: user, use: use, mime_type: mime_type) if ((io.blank? && file_set.import_url.present?) || use == Hyrax::FileMetadata::Use::THUMBNAIL)
    return version_upload(file_set: file_set, io: io, user: user) if use == Hyrax::FileMetadata::Use::ORIGINAL_FILE && file_set.original_file_id && storage_adapter.supports?(:versions)

    parent_object = file_set.parent_object
    siblings = parent_object.file_sets

    duplicate_record = siblings.select {|fs| fs.title[0]&.downcase == filename.downcase && fs.id != file_set.id }
    new_title = nil

    if duplicate_record.present?
      new_title = genrate_file_set_title(siblings, filename)
    end

    file_set.title = [new_title] if new_title.present?
    file_set.label = new_title if new_title.present?
    file_size = io.size
    mime_type ||= file_set.get_mime_type

    streamfile = storage_adapter.upload(file: io, original_filename: new_title || filename, resource: file_set)
    file_metadata = Hyrax::FileMetadata(streamfile)
    file_metadata.file_set_id = file_set.id
    file_metadata.pcdm_use = Array(use)
    file_metadata.recorded_size = Array(io.size)
    file_metadata.mime_type = mime_type if mime_type
    file_metadata.original_filename = File.basename(new_title || filename).to_s || File.basename(io)

    if use == Hyrax::FileMetadata::Use::ORIGINAL_FILE
      # Set file set label.
      reset_title = file_set.title.first == file_set.label
      # set title to label if that's how it was before this characterization
      file_set.title = file_metadata.original_filename if reset_title
      # always set the label to the original_name
      file_set.label = file_metadata.original_filename
    end

    saved_metadata = Hyrax.persister.save(resource: file_metadata)
    saved_metadata.original_filename = filename if saved_metadata.original_filename.blank?

    add_file_to_file_set(file_set: file_set,
                         file_metadata: saved_metadata,
                         user: user)

    Hyrax.publisher.publish("file.uploaded", metadata: saved_metadata)
    Hyrax.publisher.publish('file.metadata.updated', metadata: saved_metadata, user: user)

    clear_old_s3_file(file_set)
    saved_metadata
  end

  def version_upload(file_set:, io:, user:)
    file_metadata = @file_set_file_service.primary_file_for(file_set: file_set)

    Hyrax::VersioningService.create(file_metadata, user, io)
    Hyrax.publisher.publish("file.uploaded", metadata: file_metadata)
    ContentNewVersionEventJob.perform_later(file_set, user)
  end

  def build_file_metadata(filename:, file_set:, user:, use:, mime_type:)
    shrine_protocol = Valkyrie::Storage::Shrine::PROTOCOL
    file = Hyrax::UploadedFile.find_by(file_set_uri: file_set.id)
    parent_work = file_set.parent_work
    file_name = get_file_name(parent_work, file_set)

    identifier = shrine_protocol + "#{parent_work.id.to_s}/" + file_name
    file_metadata = Hyrax::FileMetadata.new(file_identifier: identifier, original_filename: file_name)
    file_metadata.file_set_id = file_set.id
    file_metadata.pcdm_use = Array(use)
    
    if use == Hyrax::FileMetadata::Use::ORIGINAL_FILE
      # Set file set label.
      reset_title = file_set.title.first == file_set.label
      # set title to label if that's how it was before this characterization
      file_set.title = file_metadata.original_filename if reset_title
      # always set the label to the original_name
      file_set.label = file_metadata.original_filename
      file_set = move_remote_file(file_set: file_set, parent_work: parent_work, filename: filename)
    end

    saved_metadata = Hyrax.persister.save(resource: file_metadata)

    add_file_to_file_set(file_set: file_set,
                         file_metadata: saved_metadata,
                         user: user)
    Hyrax.publisher.publish("file.uploaded", metadata: saved_metadata)
    Hyrax.publisher.publish('file.metadata.updated', metadata: saved_metadata, user: user)

    saved_metadata
  end

  def move_remote_file(file_set:, parent_work:, filename:)
    s3 = S3StorageService.new
    s3.init_client
    source_bucket, source_key = file_set.import_url.split('/', 2)
    target_bucket_name = s3.sanitise_name(ENV['S3_FILE_UPLOAD_BUCKET'])
    file_name = file_set.import_url.present? ? file_set.import_url.split("/data/", 2)[1] : filename
    target_object_key = "#{parent_work.id.to_s}/#{file_name}"

    s3.copy_object(source_bucket, target_bucket_name, source_key, target_object_key)
    file_set.import_url = target_object_key
    file_set
  end

  # @param [Hyrax::FileSet] file_set the file set to add to
  # @param [Hyrax::FileMetadata] file_metadata the metadata object representing
  #   the file to add
  # @param [::User] user  the user performing the add
  #
  # @return [Hyrax::FileSet] updated file set
  def add_file_to_file_set(file_set:, file_metadata:, user:)
    file_set.file_ids += [file_metadata.id]
    Hyrax.persister.save(resource: file_set)
    Hyrax.publisher.publish('object.membership.updated', object: file_set, user: user)
  end

  def genrate_file_set_title(siblings, file_name)

    existing_titles = siblings.map{ |fs| fs.title[0]&.downcase }.to_a

    base_name, extension = File.basename(file_name, ".*"), File.extname(file_name)

    suffix = 1
    new_title = file_name
    while existing_titles.include?(new_title.downcase)
      new_title = "#{base_name}_#{suffix}#{extension}"
      suffix += 1
    end

    new_title
  end

  def clear_old_s3_file(file_set)
    s3 = S3StorageService.new
    s3.init_client
    file = Hyrax::UploadedFile.find_by(file_set_uri: file_set.id)
    bucket_name = s3.sanitise_name(ENV['S3_INITIAL_FILE_UPLOAD_BUCKET'])
    object_key = file.uploader.file.path
    s3.delete_object(bucket_name, object_key)
  end

  def get_file_name(parent_work, file_set)
    if file_set.import_url.present?
      if parent_work.is_a?(Dataset)
        file_set.import_url.split("/", 2)[1]
      elsif file_set.import_url.include?("/data/")
        file_set.import_url.split("/data/", 2)[1]
      end
    else
      (file_set.label || file_set.title[0])
    end
  end
end